@@ -12,7 +12,7 @@
     
     LICENCE
     
-    FMRIB Software Library, Release 4.0 (c) 2007, The University of
+    FMRIB Software Library, Release 5.0 (c) 2012, The University of
     Oxford (the "Software")
     
     The Software remains the property of the University of Oxford ("the
@@ -61,7 +61,7 @@
     interested in using the Software commercially, please contact Isis
     Innovation Limited ("Isis"), the technology transfer company of the
     University, to negotiate a licence. Contact details are:
-    innovation@isis.ox.ac.uk quoting reference DE/1112. */
+    innovation@isis.ox.ac.uk quoting reference DE/9564. */
 
 #include "newimage/newimageall.h"
 #include "newimage/fmribmain.h"
@@ -129,7 +129,7 @@ int fmrib_main(int argc, char *argv[])
 
 int main(int argc,char *argv[])
 {
-  string title("fslcc (Version 1.2)");
+  string title("fslcc");
   string examples("fslcc [options] <first_input> <second_input> ");
   OptionParser options(title, examples);
 
