@@ -15,7 +15,7 @@
     
     LICENCE
     
-    FMRIB Software Library, Release 4.0 (c) 2007, The University of
+    FMRIB Software Library, Release 5.0 (c) 2012, The University of
     Oxford (the "Software")
     
     The Software remains the property of the University of Oxford ("the
@@ -64,7 +64,7 @@
     interested in using the Software commercially, please contact Isis
     Innovation Limited ("Isis"), the technology transfer company of the
     University, to negotiate a licence. Contact details are:
-    innovation@isis.ox.ac.uk quoting reference DE/1112. */
+    innovation@isis.ox.ac.uk quoting reference DE/9564. */
 
 #include "inference_vb.h"
 #include "newimage/newimageall.h"
@@ -109,6 +109,7 @@ public:
  protected:
 
     int spatialDims; // 0 = no spatial norm; 2 = slice only; 3 = volume
+    bool continuingFromFile;
 
     //    bool useDataDrivenSmoothness;
     //    bool useShrinkageMethod;
@@ -121,6 +122,8 @@ public:
     vector<vector<int> > neighbours; // Sparse matrix would be easier
     vector<vector<int> > neighbours2; // Sparse matrix would be easier
     void CalcNeighbours(const NEWIMAGE::volume<float>& mask);
+
+    vector<string> imagepriorstr;
     
     // For the new (Sahani-based) smoothing method:    
     CovarianceCache covar;
