@@ -15,7 +15,7 @@
     
     LICENCE
     
-    FMRIB Software Library, Release 4.0 (c) 2007, The University of
+    FMRIB Software Library, Release 5.0 (c) 2012, The University of
     Oxford (the "Software")
     
     The Software remains the property of the University of Oxford ("the
@@ -64,7 +64,11 @@
     interested in using the Software commercially, please contact Isis
     Innovation Limited ("Isis"), the technology transfer company of the
     University, to negotiate a licence. Contact details are:
-    innovation@isis.ox.ac.uk quoting reference DE/1112. */
+    innovation@isis.ox.ac.uk quoting reference DE/9564. */
+
+#ifndef EXPOSE_TREACHEROUS
+#define EXPOSE_TREACHEROUS
+#endif
 
 #include <iostream>
 #include <fstream>
@@ -650,7 +654,7 @@ public:
 	}
       }
       else{   //Do constrained optimization
-      	PVM_single_c pvm(m_data,m_bvecs,m_bvals,opts.nfibres.value(),opts.f0.value());
+      	PVM_single_c pvm(m_data,m_bvecs,m_bvals,opts.nfibres.value(),false,opts.f0.value());
 	pvm.fit(); // this will give th,ph,f in the correct order
       
 	pvmf  = pvm.get_f();
@@ -666,7 +670,7 @@ public:
 	  //If the full model gives values that are considered implausible, or we are in a CSF voxel (f1<0.05)
 	  //then fit a model without the f0 and drive f0_init to almost zero 
 	  if ((opts.nfibres.value()>0 && pvmf(1)<0.05) || pvmd>0.007 || pvmf0>0.4){
-	    PVM_single_c pvm2(m_data,m_bvecs,m_bvals,opts.nfibres.value(),false);
+	    PVM_single_c pvm2(m_data,m_bvecs,m_bvals,opts.nfibres.value(),false,false);
 	    pvm2.fit(); // this will give th,ph,f in the correct order
 	    pvmf0=0.001;
 	    pvmS0=pvm2.get_s0();
@@ -680,7 +684,7 @@ public:
 	}
       }
 
-      if(pvmd<0 || pvmd>0.01)
+      if(pvmd<0 || pvmd>0.008)
 	pvmd=2e-3;
    
       m_multifibre.set_S0(pvmS0);
@@ -726,7 +730,7 @@ public:
 	  m_multifibre.set_f0(pvmf0);
       }
 
-      if(pvmd<0 || pvmd>0.01) pvmd=2e-3;
+      if(pvmd<0 || pvmd>0.008) pvmd=2e-3;
       if(pvmd_std<0 || pvmd_std>0.01) pvmd_std=pvmd/10;
 
       m_multifibre.set_S0(pvmS0);
@@ -820,6 +824,54 @@ public:
 };
 
 
+//Get the scale-free Qform matrix to rotate bvecs back to scanner's coordinate system 
+//After applying this matrix, make sure to sign flip the x coordinate of the resulted bvecs!
+//If you apply the inverse of the matrix, sign flip the x coordinate of the input bvecs
+void Return_Qform(const Matrix& qform_mat, Matrix& QMat, const float xdim, const float ydim, const float zdim){ 
+  Matrix QMat_tmp; DiagonalMatrix Scale(3);
+   
+  QMat_tmp=qform_mat.SubMatrix(1,3,1,3);
+  Scale(1)=xdim; Scale(2)=ydim; Scale(3)=zdim;
+  QMat_tmp=Scale.i()*QMat_tmp;
+  QMat=QMat_tmp;
+}
+
+
+
+//Correct bvals/bvecs accounting for Gradient Nonlinearities
+//ColumnVector grad_nonlin has 9 entries, corresponding to the 3 components of each of the x,y and z gradient deviation
+void correct_bvals_bvecs(const Matrix& bvals,const Matrix& bvecs, const ColumnVector& grad_nonlin, const Matrix& Qform, const Matrix& Qform_inv, Matrix& bvals_c, Matrix& bvecs_c){
+  bvals_c=bvals; bvecs_c=bvecs;
+  Matrix L(3,3);  //gradient coil tensor
+  float mag;
+  L(1,1)=grad_nonlin(1);  L(1,2)=grad_nonlin(4);  L(1,3)=grad_nonlin(7);
+  L(2,1)=grad_nonlin(2);  L(2,2)=grad_nonlin(5);  L(2,3)=grad_nonlin(8);
+  L(3,1)=grad_nonlin(3);  L(3,2)=grad_nonlin(6);  L(3,3)=grad_nonlin(9);
+
+  IdentityMatrix Id(3);
+  
+  for (int l=1; l<=bvals.Ncols(); l++){
+    if (bvals(1,l)>0){ //do not correct b0s
+      //Rotate bvecs to scanner's coordinate system
+      ColumnVector bvec_tmp(3);
+      bvec_tmp=Qform*bvecs.Column(l);
+      bvec_tmp(1)=-bvec_tmp(1); //Sign-flip X coordinate
+
+      //Correct for grad-nonlin in scanner's coordinate system
+      bvecs_c.Column(l)=(Id+L)*bvec_tmp;//bvecs.Column(l);
+      mag=sqrt(bvecs_c(1,l)*bvecs_c(1,l)+bvecs_c(2,l)*bvecs_c(2,l)+bvecs_c(3,l)*bvecs_c(3,l));
+      if (mag!=0)
+	bvecs_c.Column(l)=bvecs_c.Column(l)/mag;
+      bvals_c(1,l)=mag*mag*bvals(1,l);
+      bvec_tmp=bvecs_c.Column(l);
+
+      //Rotate corrected bvecs back to voxel coordinate system
+      bvec_tmp(1)=-bvec_tmp(1); //Sign-flip X coordinate
+      bvecs_c.Column(l)=Qform_inv*bvec_tmp;
+    }
+  }
+}
+
 
 
 ////////////////////////////////////////////
@@ -850,30 +902,52 @@ int main(int argc, char *argv[])
 	bvecs(3,i)=bvecs(3,i)/tmpsum;
       }  
     }
-    
+
     volume4D<float> data;
     read_volume4D(data,opts.datafile.value());
     read_volume(mask,opts.maskfile.value());
     datam=data.matrix(mask); 
     matrix2volkey=data.matrix2volkey(mask);
     vol2matrixkey=data.vol2matrixkey(mask);
- 
-    Matrix Amat;
-    ColumnVector alpha, beta;
+    Samples samples(vol2matrixkey,matrix2volkey,datam.Ncols(),datam.Nrows());
+
+    //Read Gradient Non_linearity Maps if provided
+    volume4D<float> grad; Matrix gradm, Qform, Qform_inv;
+    if (opts.grad_file.set()){
+      read_volume4D(grad,opts.grad_file.value());
+      gradm=grad.matrix(mask);
+      //Get the scale-free Qform matrix to rotate bvecs back to scanner's coordinate system 
+      Return_Qform(data.qform_mat(), Qform, data.xdim(), data.ydim(), data.zdim());
+      Qform_inv=Qform.i();
+    }
+
+    Matrix Amat; ColumnVector alpha, beta;
     Amat=form_Amat(bvecs,bvals);
     cart2sph(bvecs,alpha,beta);
-    Samples samples(vol2matrixkey,matrix2volkey,datam.Ncols(),datam.Nrows());
   
     if(opts.rician.value() && !opts.nonlin.value()) 
       cout<<"Rician noise model requested. Non-linear parameter initialization will be performed, overriding other initialization options!"<<endl;
-  
+
     for(int vox=1;vox<=datam.Ncols();vox++){
       cout <<vox<<"/"<<datam.Ncols()<<endl;
-      xfibresVoxelManager  vm(datam.Column(vox),alpha,beta,bvecs,bvals,samples,vox);
-      vm.initialise(Amat);
-      vm.runmcmc();
+      if (!opts.grad_file.set()){
+	xfibresVoxelManager  vm(datam.Column(vox),alpha,beta,bvecs,bvals,samples,vox);
+	vm.initialise(Amat);
+	vm.runmcmc();
+      }
+      else{ //Correct for each voxel the respective bvals/bvecs
+	Matrix Amat_c, bvals_c, bvecs_c;
+	ColumnVector alpha_c, beta_c;
+	
+	correct_bvals_bvecs(bvals,bvecs, gradm.Column(vox),Qform,Qform_inv,bvals_c,bvecs_c); //correct for gradient nonlinearities
+	Amat_c=form_Amat(bvecs_c,bvals_c);
+	cart2sph(bvecs_c,alpha_c,beta_c);
+	xfibresVoxelManager  vm(datam.Column(vox),alpha_c,beta_c,bvecs_c,bvals_c,samples,vox);
+	vm.initialise(Amat_c);
+	vm.runmcmc();
+      }
     }
-    
+
     samples.save(mask);
 
   }
