@@ -7,8 +7,6 @@
     
     Copyright (C) 1999-2008 University of Oxford */
 
-// {{{  includes/namespaces
-
 /*  Part of FSL - FMRIB's Software Library
     http://www.fmrib.ox.ac.uk/fsl
     fsl@fmrib.ox.ac.uk
@@ -20,7 +18,7 @@
     
     LICENCE
     
-    FMRIB Software Library, Release 4.0 (c) 2007, The University of
+    FMRIB Software Library, Release 5.0 (c) 2012, The University of
     Oxford (the "Software")
     
     The Software remains the property of the University of Oxford ("the
@@ -69,7 +67,7 @@
     interested in using the Software commercially, please contact Isis
     Innovation Limited ("Isis"), the technology transfer company of the
     University, to negotiate a licence. Contact details are:
-    innovation@isis.ox.ac.uk quoting reference DE/1112. */
+    innovation@isis.ox.ac.uk quoting reference DE/9564. */
 
 #include "newimage/newimageall.h"
 #include "meloptions.h"
@@ -82,31 +80,45 @@
 
 using namespace Utilities;
 using namespace NEWIMAGE;
-
-// }}}
  
 namespace Melodic{
 // {{{ Setup
 
-  Matrix MelodicData::process_file(string fname, int numfiles)
+  ReturnMatrix MelodicData::process_file(string fname, int numfiles)
   {
-    volume4D<float> RawData;
+	Matrix tmpData;
+	{
+    	volume4D<float> RawData;
 
-    //read data
-    message("Reading data file " << fname << "  ... ");
-    read_volume4D(RawData,fname);
-    message(" done" << endl);
-    del_vols(RawData,opts.dummy.value());
+		if(opts.debug.value())
+			memmsg(" before reading file "<< fname);
+
+    	//read data
+    	message("Reading data file " << fname << "  ... ");
+    	read_volume4D(RawData,fname);
+    	message(" done" << endl);
+    	if(opts.debug.value())
+			memmsg(" after reading file "<< fname);
+		
+		del_vols(RawData,opts.dummy.value());
     
-    Mean += meanvol(RawData)/numfiles;
+    	Mean += meanvol(RawData)/numfiles;
+
+		//estimate smoothness
+		if(opts.debug.value())
+			memmsg(" before est smoothness ");
+    	if((Resels == 0)&&(!opts.filtermode))
+      		Resels = est_resels(RawData,Mask);
+		if(opts.debug.value())
+			memmsg(" after smoothness ");
 	
-    //reshape
-    Matrix tmpData;
-    tmpData = RawData.matrix(Mask);
-    
-    //estimate smoothness
-    if((Resels == 0)&&(!opts.filtermode))
-      Resels = est_resels(RawData,Mask);
+    	//reshape
+		if(opts.debug.value())
+			memmsg(" before reshape ");
+    	tmpData = RawData.matrix(Mask);
+    	if(opts.debug.value())
+			memmsg(" after reshape ");	  
+	}    
         
     //convert to percent BOLD signal change
     if(opts.pbsc.value()){
@@ -119,16 +131,18 @@ namespace Melodic{
 		if(opts.remove_meanvol.value())
 		{	      
 			message(string("  Removing mean image ..."));
-      		meanR = mean(tmpData);
-      		tmpData = remmean(tmpData);
+      		if(opts.debug.value())
+				memmsg(" before remmean ");
+      		remmean(tmpData,meanR,1);
+      		if(opts.debug.value())
+				memmsg(" after remmean ");
       		message(" done" << endl);
 		}
 		else meanR=ones(1,tmpData.Ncols());
     }
 
 	if(opts.remove_meantc.value()){
-    	meanC = mean(tmpData,2);
-		tmpData = remmean(tmpData,2);
+		remmean(tmpData,meanC,2);
 	}
 	
     //convert to power spectra
@@ -150,6 +164,8 @@ namespace Melodic{
     }
       
     //variance - normalisation
+   	if(opts.debug.value())
+		memmsg(" before VN ");
     if(opts.varnorm.value()){
       message("  Normalising by voxel-wise variance ..."); 
 			if(stdDev.Storage()==0)
@@ -159,9 +175,12 @@ namespace Melodic{
 				stdDev += varnorm(tmpData,std::min(30,tmpData.Nrows()-1),
 					opts.vn_level.value())/numfiles;
       stdDevi = pow(stdDev,-1); 
+	  if(opts.debug.value())
+	    memmsg(" in VN ");
       message(" done" << endl);
     }
 
+	tmpData.Release();
     return tmpData;
   }
 
@@ -202,54 +221,85 @@ namespace Melodic{
 
   void MelodicData::set_TSmode()
   {
-    Matrix tmp, tmpT, tmpS, tmpT2, tmpS2, tmpT3;
+   	dbgmsg(string("START: set_TSmode"));	
+	
+	Matrix tmp, tmpT, tmpS, tmpT2, tmpS2, tmpT3;
+	
     tmp = expand_dimred(mixMatrix);
     tmpT = zeros(tmp.Nrows()/numfiles, tmp.Ncols());
-    tmpS = zeros(numfiles, tmp.Ncols());
-    explained_var = krfact(tmp,tmpT,tmpS);
-		outMsize("tmp",tmp);
-		outMsize("tmpT",tmpT);
-		outMsize("tmpS",tmpS);
-		
-    Tmodes.clear(); Smodes.clear();
-    for(int ctr = 1; ctr <= tmp.Ncols(); ctr++){
-			tmpT3 << reshape(tmp.Column(ctr),tmpT.Nrows(),numfiles);
-			outMsize("tmpT3", tmpT3);
-      tmpT2 << tmpT.Column(ctr);
-      tmpS2 << tmpS.Column(ctr);
-			tmpT3 << SP(tmpT3,pow(ones(tmpT3.Nrows(),1)*tmpS2.t(),-1));
-			if(numfiles>1)
-				tmpT2 |= tmpT3;
-			if(mean(tmpS2,1).AsScalar()<0){
-				tmpT2*=-1.0;
-				tmpS2*=-1.0;
-			}
-      add_Tmodes(tmpT2);
-      add_Smodes(tmpS2);
-    }
+    tmpS = ones(numfiles, tmp.Ncols());
 
-	//add GLM OLS fit
-	if(Tdes.Storage()){
-		Matrix alltcs = Tmodes.at(0).Column(1);
-		for(int ctr=1; ctr < (int)Tmodes.size();ctr++)
-			alltcs|=Tmodes.at(ctr).Column(1);
-		if((alltcs.Nrows()==Tdes.Nrows())&&(Tdes.Nrows()>Tdes.Ncols()))
-			glmT.olsfit(alltcs,Tdes,Tcon);
+	outMsize("tmp",tmp);
+	outMsize("tmpT",tmpT);
+	outMsize("tmpS",tmpS);
+
+  	dbgmsg(string("   approach ") << opts.approach.value() << endl);	
+
+	if(opts.approach.value()==string("tica")){
+      message("Calculating T- and S-modes " << endl);
+      explained_var = krfact(tmp,tmpT,tmpS);
+      Tmodes.clear(); Smodes.clear();
+      for(int ctr = 1; ctr <= tmp.Ncols(); ctr++){
+	    tmpT3 << reshape(tmp.Column(ctr),tmpT.Nrows(),numfiles);
+		outMsize("tmpT3", tmpT3);
+      	tmpT2 << tmpT.Column(ctr);
+      	tmpS2 << tmpS.Column(ctr);
+		tmpT3 << SP(tmpT3,pow(ones(tmpT3.Nrows(),1)*tmpS2.t(),-1));
+		if(numfiles>1)
+	      tmpT2 |= tmpT3;
+		if(mean(tmpS2,1).AsScalar()<0){
+		  tmpT2*=-1.0;
+		  tmpS2*=-1.0;
+		}
+      	add_Tmodes(tmpT2);
+      	add_Smodes(tmpS2);
+	  }
 	}
-	if(Sdes.Storage()){
-		Matrix alltcs = Smodes.at(0);
-		for(int ctr=1; ctr < (int)Smodes.size();ctr++)
-			alltcs|=Smodes.at(ctr);
-		if((alltcs.Nrows()==Sdes.Nrows())&&(Sdes.Nrows()>Sdes.Ncols()&&alltcs.Nrows()>2))
-			glmS.olsfit(alltcs,Sdes,Scon);
+	else{
+		Tmodes.clear();
+		Smodes.clear();
+		for(int ctr = 1; ctr <= tmp.Ncols(); ctr++){
+			tmpT3 << tmp.Column(ctr);
+			add_Tmodes(tmpT3);
+		}
 	}
+	
+    if(opts.approach.value()!=string("concat")){
+	  //add GLM OLS fit
+	  dbgmsg(string(" GLM fitting ") << endl);
+
+	  if(Tdes.Storage()){
+	    Matrix alltcs = Tmodes.at(0).Column(1);
+	    for(int ctr=1; ctr < (int)Tmodes.size();ctr++)
+		  alltcs|=Tmodes.at(ctr).Column(1);
+	    if((alltcs.Nrows()==Tdes.Nrows())&&(Tdes.Nrows()>Tdes.Ncols()))
+		  glmT.olsfit(alltcs,Tdes,Tcon);
+	  }
+	  if(Sdes.Storage()){
+	    Matrix alltcs = Smodes.at(0);
+	    for(int ctr=1; ctr < (int)Smodes.size();ctr++)
+	  	  alltcs|=Smodes.at(ctr);
+	    if((alltcs.Nrows()==Sdes.Nrows())&&(Sdes.Nrows()>Sdes.Ncols()&&alltcs.Nrows()>2))
+		  glmS.olsfit(alltcs,Sdes,Scon);
+	  }
+		
+    }
+  //    else{
+//		dbgmsg(string(" Bypassing krfac ") << endl);
+//        add_Tmodes(tmp);
+//		add_Smodes(tmpS);
+//      }
+	
+	dbgmsg(string("END: set_TSmode"));	
   }
 
   void MelodicData::setup()
   { 
     numfiles = (int)opts.inputfname.value().size();
     setup_misc();
-
+	if(opts.debug.value())
+		memmsg(" after setup_misc ");
+		
 	if(opts.filtermode){ // basic setup for filtering only
 		Data = process_file(opts.inputfname.value().at(0));
 	}
@@ -265,7 +315,9 @@ namespace Melodic{
 			opts.varnorm.set_T(false);
 		}
     	alldat = process_file(opts.inputfname.value().at(0), numfiles) / numfiles;
-
+		if(opts.debug.value())
+			memmsg(" after process_file ");
+			
 		if(opts.pca_dim.value() > alldat.Nrows()-2){
 			cerr << "ERROR:: too many components selected \n\n";
 			exit(2);
@@ -274,14 +326,27 @@ namespace Melodic{
  		if(opts.debug.value())
 			save4D(alldat,string("preproc_dat") + num2str(1));   
 		for(int ctr = 1; ctr < numfiles; ctr++){
-    		tmpData = process_file(opts.inputfname.value().at(ctr), numfiles);
+    		tmpData = process_file(opts.inputfname.value().at(ctr), numfiles) / numfiles;
 			if(opts.debug.value())
-				save4D(tmpData /numfiles,string("preproc_dat") + num2str(ctr+1));
+				save4D(tmpData,string("preproc_dat") + num2str(ctr+1));
 			if(tmpData.Ncols() == alldat.Ncols() && tmpData.Nrows() == alldat.Nrows())
-      			alldat += tmpData / numfiles;	
-			else
-				message("Data dimensions do not match - ignoring "+opts.inputfname.value().at(ctr) << endl);
-   		}
+      			alldat += tmpData;	
+			else{
+				    if(opts.approach.value()==string("tica")){
+						cerr << "ERROR:: data dimensions do not match, TICA not possible \n\n";
+						exit(2); 
+					}
+					
+					if(tmpData.Ncols() == alldat.Ncols()){
+						int mindim = min(alldat.Nrows(),tmpData.Nrows());
+						alldat = alldat.Rows(1,mindim);
+						tmpData = tmpData.Rows(1,mindim);
+						alldat += tmpData;	
+					}				
+					else 	
+					message("Data dimensions do not match - ignoring "+opts.inputfname.value().at(ctr) << endl);
+				}
+   		}	
 
     	//update mask
     	if(opts.update_mask.value()){
@@ -312,12 +377,17 @@ namespace Melodic{
     	order = ppca_dim(remmean(alldat,2), RXweight, tmpPPCA, AdjEV, PercEV, Corr, pcaE, pcaD, Resels, opts.pca_est.value());	  
 		if (opts.paradigmfname.value().length()>0)
 			order += param.Ncols();
-			
+
 	  	if(opts.pca_dim.value() == 0){
       		opts.pca_dim.set_T(order);
 			PPCA=tmpPPCA;
   		}
+	  	if(opts.pca_dim.value() < 0){
+      		opts.pca_dim.set_T(min(order,-1*opts.pca_dim.value()));
+			PPCA=tmpPPCA;
+  		}
     	order = opts.pca_dim.value();
+		dbgmsg(endl << "Model order : "<<order<<endl);
 
 		if (opts.paradigmfname.value().length()>0){
 			Matrix tmpPscales;
@@ -336,7 +406,6 @@ namespace Melodic{
 			outMsize("tmpPPCA",tmpPPCA); saveascii(tmpPPCA,"tmpPPCA");
 			outMsize("whiteMatrix",whiteMatrix); saveascii(whiteMatrix,"whiteMatrix");
 			outMsize("dewhiteMatrix",dewhiteMatrix); saveascii(dewhiteMatrix,"dewhiteMatrix");
-			cerr << "Order: " << order << endl;
 		}
 
 		EV = AdjEV;
@@ -349,12 +418,16 @@ namespace Melodic{
       		WM.push_back(tmp);
     	} 
 		else {
-	
-      		for(int ctr = 0; ctr < numfiles; ctr++){
+  
+		dbgmsg("Multi-Subject ICA");
+  		for(int ctr = 0; ctr < numfiles; ctr++){
 				tmpData = process_file(opts.inputfname.value().at(ctr), numfiles);
 	
 				if(opts.joined_vn.value() && tmpvarnorm){
-					tmpData=SP(tmpData,pow(ones(tmpData.Nrows(),1)*stdDev,-1));
+					dbgmsg("tmpData normalisation"<< endl);
+					dbgmsg("stdDev "  << stdDev(1,2)<< endl);
+					dbgmsg("tmpData " << tmpData.SubMatrix(1,1,1,2)<< endl);
+					SP3(tmpData,pow(stdDev,-1));
 				}
 				//  whiten (separate / joint)
 				Matrix newWM,newDWM; 
@@ -371,10 +444,11 @@ namespace Melodic{
 					}
 					else{
 					  if(opts.debug.value())
-					    message(" --mod_pca ");
+					    dbgmsg(" --mod_pca ");
 						Matrix tmp1, tmp2;
 						tmp1 = whiteMatrix * alldat;
-						tmp1 = remmean(tmp1,2) * tmpData.t();
+						remmean(tmp1,2);
+						tmp1 *= tmpData.t();
 						tmp2 = pinv(tmp1.t()).t();  
 						std_pca(tmp1 * tmpData, RXweight, Corr, pcaE, pcaD);
 						calc_white(pcaE, pcaD, order, newWM, newDWM);		
@@ -407,6 +481,7 @@ namespace Melodic{
     }
   } // void setup()
 	
+
   void MelodicData::setup_misc()
   {
 
@@ -485,9 +560,10 @@ namespace Melodic{
 			Scon &= -1*Scon;
 		}
 	}
-	Tdes = remmean(Tdes,1);
+	remmean(Tdes);
   }
 
+
   void MelodicData::save()
   {   
 
@@ -548,6 +624,7 @@ namespace Melodic{
       }
      
     //Output T- & S-modes
+ 
     save_Tmodes();
     save_Smodes();
 
@@ -606,6 +683,7 @@ namespace Melodic{
 		message("...done" << endl);
   } //void save()
 
+
   int MelodicData::remove_components()
   {  
     message("Reading " << opts.filtermix.value() << endl) 
@@ -680,6 +758,7 @@ namespace Melodic{
     return 0;
   } // int remove_components()
 
+
   void MelodicData::create_RXweight()
   {
     message("Reading the weights for the covariance R_X from file "<< opts.segment.value() << endl);
@@ -689,6 +768,7 @@ namespace Melodic{
     RXweight = tmpRX.matrix(Mask);
   } 
 
+
   void MelodicData::est_smoothness()
   {
     if(Resels == 0){
@@ -725,6 +805,7 @@ namespace Melodic{
     }
   }
 
+
   unsigned long MelodicData::standardise(volume<float>& mask, volume4D<float>& R)
   {
     	unsigned long count = 0;
@@ -766,6 +847,7 @@ namespace Melodic{
     	return count;
   }
 
+
   float MelodicData::est_resels(volume4D<float> R, volume<float> mask)
   {
     message("  Estimating data smoothness ... ");
@@ -841,6 +923,7 @@ namespace Melodic{
     return resels;
   }
 
+
   void MelodicData::create_mask(volume<float>& theMask)
   {
     if(opts.use_mask.value() && opts.maskfname.value().size()>0){   // mask provided 
@@ -915,6 +998,7 @@ namespace Melodic{
     }
   } //void create_mask()
 
+
   void MelodicData::sort()
   {
     int numComp = mixMatrix.Ncols(), numVox = IC.Ncols(), 
@@ -930,7 +1014,7 @@ namespace Melodic{
 	  }
     message("Sorting IC maps" << endl);  
     Matrix tmpscales, tmpICrow, tmpMIXcol;
-	if(numfiles > 1){
+	if(numfiles > 1 && opts.approach.value()==string("tica")){
 		set_TSmode();
 		Matrix allmodes = Smodes.at(0);
 		for(int ctr = 1; ctr < (int)Smodes.size();++ctr)
@@ -1021,3 +1105,6 @@ namespace Melodic{
 
 }
 
+
+
+
