@@ -18,7 +18,7 @@
     
     LICENCE
     
-    FMRIB Software Library, Release 4.0 (c) 2007, The University of
+    FMRIB Software Library, Release 5.0 (c) 2012, The University of
     Oxford (the "Software")
     
     The Software remains the property of the University of Oxford ("the
@@ -67,7 +67,7 @@
     interested in using the Software commercially, please contact Isis
     Innovation Limited ("Isis"), the technology transfer company of the
     University, to negotiate a licence. Contact details are:
-    innovation@isis.ox.ac.uk quoting reference DE/1112. */
+    innovation@isis.ox.ac.uk quoting reference DE/9564. */
 
 #ifndef __MELGMIX_h
 #define __MELGMIX_h
@@ -205,6 +205,14 @@ namespace Melodic{
         probmap = tempVol.matrix(Mask);
       }
 
+	  inline Matrix get_params(){
+		Matrix tmp = zeros(3,means.Ncols());
+		tmp.Row(1) = means;
+		tmp.Row(2) = vars;
+		tmp.Row(3) = props;
+		return tmp;
+      }
+
       double datamean;
       double datastdev;
 
