@@ -18,7 +18,7 @@
     
     LICENCE
     
-    FMRIB Software Library, Release 4.0 (c) 2007, The University of
+    FMRIB Software Library, Release 5.0 (c) 2012, The University of
     Oxford (the "Software")
     
     The Software remains the property of the University of Oxford ("the
@@ -67,7 +67,7 @@
     interested in using the Software commercially, please contact Isis
     Innovation Limited ("Isis"), the technology transfer company of the
     University, to negotiate a licence. Contact details are:
-    innovation@isis.ox.ac.uk quoting reference DE/1112. */
+    innovation@isis.ox.ac.uk quoting reference DE/9564. */
 
 #include "melhlprfns.h"
 #include "libprob.h"
@@ -82,7 +82,6 @@ namespace Melodic{
     Matrix DStDev=stdev(Data);
     volume4D<float> tmpMask, RawData;
     tmpMask.setmatrix(DStDev,mask);
-    RawData.setmatrix(Data,mask);
 
     float tMmax;
     volume<float> tmpMask2;
@@ -92,9 +91,23 @@ namespace Melodic{
     double st_mean = DStDev.Sum()/DStDev.Ncols();
     double st_std  = stdev(DStDev.t()).AsScalar();
       
-    mask = binarise(tmpMask2,(float) max((float) st_mean-3*st_std,(float) 0.01*st_mean),tMmax);
+	volume4D<float> newmask;
+    newmask = binarise(tmpMask2,(float) max((float) st_mean-3*st_std,(float) 0.01*st_mean),tMmax);
 
-    Data = RawData.matrix(mask);
+	Matrix newmaskM,newData;
+	newmaskM = newmask.matrix(mask);
+	int N = Data.Nrows();
+	
+	if(int(newmaskM.Row(1).SumAbsoluteValue() + 0.3) < newmaskM.Ncols()){
+		RawData.setmatrix(Data.Row(1),mask);
+		newData = RawData.matrix(newmask[0]);
+		for(int r=2; r <= N; r++){
+			RawData.setmatrix(Data.Row(r),mask);
+			newData &= RawData.matrix(newmask[0]);
+		}
+		Data = newData;
+		mask = newmask[0];  
+	}
   }
 
   void del_vols(volume4D<float>& in, int howmany)
@@ -183,12 +196,14 @@ namespace Melodic{
 
 	void varnorm(Matrix& in, const RowVector& vars)
 	{
-		Matrix tmp = vars;
-		in = SP(in,pow(ones(in.Nrows(),1) * tmp,-1));
+		for(int ctr=1; ctr <=in.Nrows();ctr++){
+			in.Row(ctr) = SD(in.Row(ctr),vars);
+		}
 	}
 	
   RowVector varnorm(Matrix& in, Matrix& Corr, int dim, float level)
   { 
+	
     Matrix tmpE, white, dewhite;
     RowVector tmpD, tmpD2;
 
@@ -207,6 +222,7 @@ namespace Melodic{
 				in.Column(ctr1) = 0.0*in.Column(ctr1);
       }
 	  varnorm(in,tmpD);
+
     return tmpD;
   }  //RowVector varnorm
 
@@ -227,7 +243,14 @@ namespace Melodic{
       Res = SP(in,Res);
     }
     return Res;
-  }  //Matrix SP
+  }  //Matrix SP2
+
+  void SP3(Matrix& in, const Matrix& weights)
+  {
+	for(int ctr=1; ctr <= in.Nrows(); ctr++){
+		in.Row(ctr) << SP(in.Row(ctr),weights.AsRow());
+	} 
+  }
 
   Matrix corrcoef(const Matrix& in1, const Matrix& in2){
 		Matrix tmp = in1;
@@ -944,7 +967,7 @@ namespace Melodic{
   }  //Matrix gen_arCorr
 
 	void basicGLM::olsfit(const Matrix& data, const Matrix& design, 
-		const Matrix& contrasts, int DOFadjust)
+		const Matrix& contrasts, int requestedDOF)
 	{
 		beta = zeros(design.Ncols(),1); 
 		residu = zeros(1); sigsq = -1.0*ones(1); varcb = -1.0*ones(1); 
@@ -958,8 +981,9 @@ namespace Melodic{
 			
 			beta = pinvdes * dat;
 			residu = dat - design*beta;
-
-			dof = design.Nrows() - design.Ncols()-1;
+			dof = ols_dof(design);
+			if ( requestedDOF>0)
+			  dof = requestedDOF;
 			sigsq = sum(SP(residu,residu))/dof;
 			
 			float fact = float(dof) / design.Ncols();
@@ -967,8 +991,7 @@ namespace Melodic{
 		
 			pf_fmf = f_fmf.Row(1); 
 			for(int ctr1=1;ctr1<=f_fmf.Ncols();ctr1++)
-				pf_fmf(1,ctr1) = 1.0-MISCMATHS::fdtr(design.Ncols(),
-				int(design.Nrows() -1 -design.Ncols()),f_fmf.Column(ctr1).AsScalar());
+				pf_fmf(1,ctr1) = 1.0-MISCMATHS::fdtr(design.Ncols(),dof,f_fmf.Column(ctr1).AsScalar());
 				
 			if(contrasts.Storage()>0 && contrasts.Ncols()==beta.Nrows()){
 				cbeta = contrasts*beta;
