@@ -18,7 +18,7 @@
     
     LICENCE
     
-    FMRIB Software Library, Release 4.0 (c) 2007, The University of
+    FMRIB Software Library, Release 5.0 (c) 2012, The University of
     Oxford (the "Software")
     
     The Software remains the property of the University of Oxford ("the
@@ -67,7 +67,7 @@
     interested in using the Software commercially, please contact Isis
     Innovation Limited ("Isis"), the technology transfer company of the
     University, to negotiate a licence. Contact details are:
-    innovation@isis.ox.ac.uk quoting reference DE/1112. */
+    innovation@isis.ox.ac.uk quoting reference DE/9564. */
 
 #ifndef __MELODICHLPR_h
 #define __MELODICHLPR_h
@@ -76,6 +76,22 @@
 #include "newmatap.h"
 #include "newmatio.h"
 
+	#ifdef __APPLE__
+	#include <mach/mach.h>
+	#define mmsg(msg) { \
+	  struct task_basic_info t_info; \
+	  mach_msg_type_number_t t_info_count = TASK_BASIC_INFO_COUNT; \
+	  if (KERN_SUCCESS == task_info(mach_task_self(), TASK_BASIC_INFO, (task_info_t) &t_info, &t_info_count)) \
+		{ \
+			cout << " MEM: " << msg << " res: " << t_info.resident_size/1000000 << " virt: " << t_info.virtual_size/1000000 << "\n"; \
+			} \
+	}
+	#else
+	#define mmsg(msg) { \
+	   cout << msg; \
+	}
+	#endif
+
 using namespace NEWIMAGE;
 
 namespace Melodic{
@@ -93,14 +109,15 @@ namespace Melodic{
   RowVector varnorm(Matrix& in, Matrix& Corr, int dim = 30, float level = 1.6);
 
   Matrix SP2(const Matrix& in, const Matrix& weights, bool econ = 0);
+  void SP3(Matrix& in, const Matrix& weights);
 
   RowVector Feta(int n1,int n2);
   RowVector cumsum(const RowVector& Inp);
 
   Matrix corrcoef(const Matrix& in1, const Matrix& in2);
   Matrix corrcoef(const Matrix& in1, const Matrix& in2, const Matrix& part);
-  Matrix calc_corr(const Matrix& in, bool econ = 0);
-  Matrix calc_corr(const Matrix& in, const Matrix& weights, bool econ = 0);
+  Matrix calc_corr(const Matrix& in, bool econ = 1);
+  Matrix calc_corr(const Matrix& in, const Matrix& weights, bool econ = 1);
 
   float calc_white(const Matrix& tmpE, const RowVector& tmpD, const RowVector& PercEV, int dim, Matrix& param, Matrix& paramS, Matrix& white, Matrix& dewhite);
   float calc_white(const Matrix& tmpE, const RowVector& tmpD, const RowVector& PercEV, int dim, Matrix& white, Matrix& dewhite);
@@ -146,7 +163,7 @@ namespace Melodic{
 			~basicGLM(){}
 		
 			void olsfit(const Matrix& data, const Matrix& design, 
-				const Matrix& contrasts, int DOFadjust = 0);
+				const Matrix& contrasts, int DOFadjust = -1);
 
 			inline Matrix& get_t(){return t;}
 			inline Matrix& get_z(){return z;}
