@@ -18,7 +18,7 @@
     
     LICENCE
     
-    FMRIB Software Library, Release 4.0 (c) 2007, The University of
+    FMRIB Software Library, Release 5.0 (c) 2012, The University of
     Oxford (the "Software")
     
     The Software remains the property of the University of Oxford ("the
@@ -67,7 +67,7 @@
     interested in using the Software commercially, please contact Isis
     Innovation Limited ("Isis"), the technology transfer company of the
     University, to negotiate a licence. Contact details are:
-    innovation@isis.ox.ac.uk quoting reference DE/1112. */
+    innovation@isis.ox.ac.uk quoting reference DE/9564. */
 
 #include "newimage/newimageall.h"
 #include "utils/log.h"
@@ -167,9 +167,11 @@ namespace Melodic{
       }		
 			
       {//plot time course
-    	IChtml << "<H3> Temporal mode </H3><p>" << endl <<endl;
-    	miscplot newplot;
+    	IChtml << "<H3> Temporal mode </H3><p>" << endl <<endl;     	
+		miscplot newplot;
+		
 			Matrix tmptc = melodat.get_Tmodes(cnum-1).Column(1).t();
+
 			newplot.col_replace(0,0xFF0000);
 
 			newplot.add_label(string("IC ")+num2str(cnum)+" time course");
@@ -201,7 +203,8 @@ namespace Melodic{
 				tmptc.Row(1).Minimum()),tmptc.Row(1).Maximum()+
 				0.05*(tmptc.Row(1).Maximum()-tmptc.Row(1).Minimum()));
 			newplot.grid_swapdefault();
-	    newplot.timeseries(tmptc,
+			
+	        newplot.timeseries(tmptc,
 			  report.appendDir(string("t")+num2str(cnum)+".png"),
 			  string("Timecourse No. ")+num2str(cnum), 
 			  opts.tr.value(),150,12,1,false);
@@ -830,6 +833,7 @@ namespace Melodic{
   }
 	
 	void MelodicReport::Smode_rep(){
+	  if(melodat.get_Smodes().size()>0){
 		report << "<p><hr><b>TICA Subject/Session modes </b> <br>" << endl;
 		miscplot newplot;
 		report << "Boxplots show the relative response amplitudes across the "
@@ -849,5 +853,6 @@ namespace Melodic{
 		newplot.boxplot(allmodes,report.appendDir(string("bp_Smodes.png")),
   			string("Subject/Session modes"));
   		report << "<img BORDER=0 SRC=\"bp_Smodes.png\"><p>" << endl;
+      }
 	}
 }
