@@ -15,7 +15,7 @@
     
     LICENCE
     
-    FMRIB Software Library, Release 4.0 (c) 2007, The University of
+    FMRIB Software Library, Release 5.0 (c) 2012, The University of
     Oxford (the "Software")
     
     The Software remains the property of the University of Oxford ("the
@@ -64,12 +64,13 @@
     interested in using the Software commercially, please contact Isis
     Innovation Limited ("Isis"), the technology transfer company of the
     University, to negotiate a licence. Contact details are:
-    innovation@isis.ox.ac.uk quoting reference DE/1112. */
+    innovation@isis.ox.ac.uk quoting reference DE/9564. */
 
 // Miscellaneous maths functions
 #define NOMINMAX
 #include <cstdlib>
 #include <cmath>
+#include <algorithm>
 #include "miscmaths.h"
 #include "miscprob.h"
 #include "stdlib.h"
@@ -99,7 +100,6 @@ namespace MISCMATHS {
   }
 
 
-
   float Sinc(const float x) {
     if (fabs(x)<1e-9) {
       return 1-x*x*M_PI*M_PI/6.0;
@@ -354,10 +354,12 @@ namespace MISCMATHS {
   
   int write_ascii_matrix(const Matrix& mat, ofstream& fs, int precision)
   {
+    fs.setf(ios::floatfield);  // use fixed or scientific notation as appropriate
     if (precision>0)  { 
-      fs.setf(ios::scientific | ios::showpos); 
       fs.precision(precision); 
-    } 
+    } else {
+      fs.precision(10);    // default precision
+    }
 #ifdef PPC64	
     int n=0;
 #endif
@@ -570,10 +572,13 @@ namespace MISCMATHS {
       }
     }
 
-  ReturnMatrix pinv(const Matrix& mat)
+  ReturnMatrix pinv(const Matrix& mat2)
     {
       // calculates the psuedo-inverse using SVD
       // note that the right-pinv(x') = pinv(x).t()
+      Matrix mat(mat2);
+      if ( mat2.Ncols() > mat2.Nrows() )
+	mat=mat.t();
       Tracer tr("pinv");
       DiagonalMatrix D;
       Matrix U, V;
@@ -585,6 +590,8 @@ namespace MISCMATHS {
 	else D(n,n) = 0.0; // reduce the number of columns because too close to singular
       }
       Matrix pinv = V * D * U.t();
+      if ( mat2.Ncols() > mat2.Nrows() )
+	pinv=pinv.t();
       pinv.Release();
       return pinv;
     }
@@ -678,6 +685,53 @@ namespace MISCMATHS {
     }
 
 
+  vector<int> get_sortindex(const Matrix& vals, const string& mode, int col)
+  {
+    // mode is either "new2old" or "old2new"
+    // return the mapping of old and new indices in the *ascending* sort of vals (from column=col)
+    int length=vals.Nrows();
+    vector<pair<double, int> > sortlist(length);
+    for (int n=0; n<length; n++) {
+      sortlist[n] = pair<double, int>((double) vals(n+1,col),n+1);
+    }
+    sort(sortlist.begin(),sortlist.end());  // O(N.log(N))
+    vector<int> idx(length);
+    for (int n=0; n<length; n++) {
+      if (mode=="old2new") {
+	//  here idx[n] is the where in the ordered list the old n'th row is mapped to (i.e. idx[n] = rank)
+	idx[sortlist[n].second-1] = n+1;
+      } else if (mode=="new2old") {
+	// here idx[n] is the the old row number of the n'th ordered item (i.e. idx[n] is old row number with rank = n)
+	idx[n] = sortlist[n].second;  
+      } else {
+	cerr << "ERROR:: unknown mode in get_sortidx = " << mode << endl; 
+      }
+    }
+    return idx;
+  }
+
+  Matrix apply_sortindex(const Matrix& vals, vector<int> sidx, const string& mode)
+  {
+    // mode is either "new2old" or "old2new"
+    // apply the index mapping from get_sortindex to the whole matrix (swapping rows)
+    Matrix res(vals);
+    res=0.0;
+    int ncols=vals.Ncols();
+    for (unsigned int n=0; n<sidx.size(); n++) {
+      int row = sidx[n];
+      if (mode=="old2new") {
+	res.SubMatrix(row,row,1,ncols)=vals.SubMatrix(n+1,n+1,1,ncols);
+      } else if (mode=="new2old") {
+	res.SubMatrix(n+1,n+1,1,ncols)=vals.SubMatrix(row,row,1,ncols);
+      } else {
+	cerr << "ERROR:: unknown mode in apply_sortidx = " << mode << endl; 
+      }
+    }
+    return res;
+  }
+
+
+
   //------------------------------------------------------------------------//
 
   // Handy MATLAB-like functions
@@ -1203,9 +1257,8 @@ float interp1(const ColumnVector& x, const ColumnVector& y, float xi)
     if(xi <= x.Minimum()) 
       ans = y(1); 
     else{
-      int ind=1;
-      while(xi >= x(ind))
-	ind++;      
+      int ind=2;
+      while(xi >= x(ind)) { ind++; }
       float xa = x(ind-1), xb = x(ind), ya = y(ind-1), yb = y(ind);
       ans = ya + (xi - xa)/(xb - xa) * (yb - ya);
     }
@@ -1419,6 +1472,15 @@ ReturnMatrix abs(const Matrix& mat)
   return res;
 }
 
+void abs_econ(Matrix& mat)
+{
+	for (int mc=1; mc<=mat.Ncols(); mc++) {
+	    for (int mr=1; mr<=mat.Nrows(); mr++) {
+	      mat(mr,mc)=std::abs(mat(mr,mc));
+	    }
+	}
+}
+
 ReturnMatrix sqrt(const Matrix& mat)
 {
   Matrix res = mat;
@@ -1437,6 +1499,21 @@ ReturnMatrix sqrt(const Matrix& mat)
   return res;
 }
 
+void sqrt_econ(Matrix& mat)
+{
+  bool neg_flag = false;
+  for (int mc=1; mc<=mat.Ncols(); mc++) {
+    for (int mr=1; mr<=mat.Nrows(); mr++) {
+      if(mat(mr,mc)<0){ neg_flag = true; }
+      mat(mr,mc)=std::sqrt(std::abs(mat(mr,mc)));
+    }
+  }
+  if(neg_flag){
+    //cerr << " Matrix contained negative elements" << endl;
+    //cerr << " return sqrt(abs(X)) instead" << endl;
+  }
+}
+
 ReturnMatrix sqrtm(const Matrix& mat)
 {
 	Matrix res, tmpU, tmpV;
@@ -1465,6 +1542,21 @@ ReturnMatrix log(const Matrix& mat)
   return res; 
 }
 
+void log_econ(Matrix& mat)
+{
+  bool neg_flag = false;
+  for (int mc=1; mc<=mat.Ncols(); mc++) {
+    for (int mr=1; mr<=mat.Nrows(); mr++) {
+      if(mat(mr,mc)<0){ neg_flag = true; }
+      mat(mr,mc)=std::log(std::abs(mat(mr,mc)));
+    }
+  }
+  if(neg_flag){
+    //  cerr << " Matrix contained negative elements" << endl;
+    //  cerr << " return log(abs(X)) instead" << endl;
+  }
+}
+
 ReturnMatrix exp(const Matrix& mat)
 {
   Matrix res = mat;
@@ -1477,6 +1569,15 @@ ReturnMatrix exp(const Matrix& mat)
   return res;
 }
 
+void exp_econ(Matrix& mat)
+{
+  for (int mc=1; mc<=mat.Ncols(); mc++) {
+    for (int mr=1; mr<=mat.Nrows(); mr++) {
+      mat(mr,mc)=std::exp(mat(mr,mc));
+    }
+  }
+}
+
   // optimised code for calculating matrix exponential
 ReturnMatrix expm(const Matrix& mat){
   float nmat = sum(mat).Maximum();
@@ -1556,6 +1657,15 @@ ReturnMatrix tanh(const Matrix& mat)
   return res;
 }
 
+void tanh_econ(Matrix& mat)
+{
+  for (int mc=1; mc<=mat.Ncols(); mc++) {
+    for (int mr=1; mr<=mat.Nrows(); mr++) {
+      mat(mr,mc)=std::tanh(mat(mr,mc));
+    }
+  }
+}
+
 ReturnMatrix pow(const Matrix& mat, const double exp)
 {
   Matrix res = mat;
@@ -1568,6 +1678,15 @@ ReturnMatrix pow(const Matrix& mat, const double exp)
   return res;
 }
 
+void pow_econ(Matrix& mat, const double exp)
+{
+  for (int mc=1; mc<=mat.Ncols(); mc++) {
+    for (int mr=1; mr<=mat.Nrows(); mr++) {
+      mat(mr,mc)=std::pow(mat(mr,mc),exp);
+    }
+  }
+}
+
 ReturnMatrix max(const Matrix& mat)
 {
   Matrix res;
@@ -1650,60 +1769,83 @@ ReturnMatrix min(const Matrix& mat)
 
 ReturnMatrix sum(const Matrix& mat, const int dim)
 {
-  Matrix tmp;
+  Matrix res;
+
+  if (dim == 1){
+		res = zeros(1,mat.Ncols());
+		for (int mc=1; mc<=mat.Ncols(); mc++) {
+		    for (int mr=1; mr<=mat.Nrows(); mr++) {
+		      	res(1,mc) += mat(mr,mc);
+		    }
+		}
+	}
+	else{
+		res = zeros(mat.Nrows(),1);
+		for (int mr=1; mr<=mat.Nrows(); mr++) {
+		    for (int mc=1; mc<=mat.Ncols(); mc++) {
+		      	res(mr,1) += mat(mr,mc);
+		    }
+		}
+	}
 
-  if (dim == 1) {tmp=mat;}
-  else {tmp=mat.t();}
-  Matrix res(1,tmp.Ncols());
-  res = 0.0;  
-  for (int mc=1; mc<=tmp.Ncols(); mc++) {
-    for (int mr=1; mr<=tmp.Nrows(); mr++) {
-      res(1,mc) += tmp(mr,mc);
-    }
-  }
-  if (!(dim == 1)) {res=res.t();}
   res.Release();
   return res;
 }
 
 ReturnMatrix mean(const Matrix& mat, const int dim)
 {
-  Matrix tmp;
-  if (dim == 1) {tmp=mat;}
-  else {tmp=mat.t();}
-
-  int N = tmp.Nrows();
-
-  Matrix res(1,tmp.Ncols());
-  res = 0.0;  
-  for (int mc=1; mc<=tmp.Ncols(); mc++) {
-    for (int mr=1; mr<=tmp.Nrows(); mr++) {
-      res(1,mc) += tmp(mr,mc)/N;
-    }
-  }
-  if (!(dim == 1)) {res=res.t();}
-
-  res.Release();
-  return res;
+	Matrix res;	
+	int N;
+  	if (dim == 1){
+		res = zeros(1,mat.Ncols());
+		N = mat.Nrows();
+		for (int mc=1; mc<=mat.Ncols(); mc++) {
+		    for (int mr=1; mr<=mat.Nrows(); mr++) {
+		      	res(1,mc) += mat(mr,mc)/N;
+		    }
+		}
+  	}
+  	else{
+		res = zeros(mat.Nrows(),1);
+		N = mat.Ncols();
+		for (int mr=1; mr<=mat.Nrows(); mr++) {
+		    for (int mc=1; mc<=mat.Ncols(); mc++) {
+		      	res(mr,1) += mat(mr,mc)/N;
+		    }
+		}
+  	}
+	res.Release();
+	return res;
 }
 
 ReturnMatrix var(const Matrix& mat, const int dim)
-{
-  Matrix tmp;
-  if (dim == 1) {tmp=mat;}
-  else {tmp=mat.t();}
-  int N = tmp.Nrows();
-  Matrix res(1,tmp.Ncols());
-  res = 0.0;
+{ 
+	Matrix res, matmean;
+	matmean = mean(mat,dim);	
+	int N;
+  	if (dim == 1){
+		res = zeros(1,mat.Ncols());
+		N = mat.Nrows();
+		if(N>1){
+			for (int mc=1; mc<=mat.Ncols(); mc++) {
+		    	for (int mr=1; mr<=mat.Nrows(); mr++) {
+		      		res(1,mc) += (mat(mr,mc) - matmean(1,mc)) * (mat(mr,mc) - matmean(1,mc))/(N-1);
+				}
+		    }
+		}
+  	}
+  	else{
+		res = zeros(mat.Nrows(),1);
+		N = mat.Ncols();
+		if(N>1){    
+			for (int mr=1; mr<=mat.Nrows(); mr++) {
+		    	for (int mc=1; mc<=mat.Ncols(); mc++) {
+		      		res(mr,1) += (mat(mr,mc) -matmean(mr,1))* (mat(mr,mc)-matmean(mr,1))/(N-1);
+				}
+		    }
+		}
+  	}
 
-  if(N>1){    
-    tmp -= ones(tmp.Nrows(),1)*mean(tmp,1);   
-    for (int mc=1; mc<=tmp.Ncols(); mc++) 
-      for (int mr=1; mr<=tmp.Nrows(); mr++) 
-        res(1,mc) += tmp(mr,mc) / (N-1) * tmp(mr,mc);
-  }
-
-  if (!(dim == 1)) {res=res.t();}
   res.Release();
   return res;
 }
@@ -1870,6 +2012,37 @@ ReturnMatrix SD(const Matrix& mat1,const Matrix& mat2)
   return ret;
 }
 
+void SD_econ(Matrix& mat1,const Matrix& mat2) 
+{
+  if((mat1.Nrows() != mat2.Nrows()) ||
+     (mat1.Ncols() != mat2.Ncols()) ){
+    cerr <<"MISCMATHS::SD - matrices are of different dimensions"<<endl;
+    exit(-1);
+  }
+  for (int r = 1; r <= mat1.Nrows(); r++) {
+    for (int c =1; c <= mat1.Ncols(); c++) {
+      if( mat2(r,c)==0)
+	mat1(r,c)=0;
+      else
+	mat1(r,c) = mat1(r,c)/mat2(r,c);
+    }
+  }
+}
+
+void SP_econ(Matrix& mat1,const Matrix& mat2) 
+{
+  if((mat1.Nrows() != mat2.Nrows()) ||
+     (mat1.Ncols() != mat2.Ncols()) ){
+    cerr <<"MISCMATHS::SD - matrices are of different dimensions"<<endl;
+    exit(-1);
+  }
+  for (int r = 1; r <= mat1.Nrows(); r++) {
+    for (int c =1; c <= mat1.Ncols(); c++) {
+		mat1(r,c) = mat1(r,c) * mat2(r,c);
+    }
+  }
+}
+
 //Deprecate?
 ReturnMatrix vox_to_vox(const ColumnVector& xyz1,const ColumnVector& dims1,const ColumnVector& dims2,const Matrix& xfm){
   ColumnVector xyz1_mm(4),xyz2_mm,xyz2(3);
@@ -1892,43 +2065,40 @@ ReturnMatrix mni_to_imgvox(const ColumnVector& mni,const ColumnVector& mni_origi
   return img_vox;
 }
 
+void remmean_econ(Matrix& mat, const int dim)
+{
+	Matrix matmean;
+	remmean(mat, matmean, dim);
+}
+
+void remmean(Matrix& mat, Matrix& matmean, const int dim)
+{
+	matmean = mean(mat,dim);
+  	if (dim == 1){
+ 		for (int mr=1; mr<=mat.Nrows(); mr++)
+			mat.Row(mr) -= matmean.AsRow();
+  	}
+  	else{
+		for (int mc=1; mc<=mat.Ncols(); mc++)
+			mat.Column(mc) -= matmean.AsColumn(); 
+  	}
+}
+
+void remmean(const Matrix& mat, Matrix& demeanedmat, Matrix& Mean,  const int dim)
+{ 
+	demeanedmat = mat;
+	remmean(demeanedmat,Mean, dim);
+}
 
 ReturnMatrix remmean(const Matrix& mat, const int dim)
 { 
-  Matrix res;
-  if (dim == 1) {res=mat;}
-  else {res=mat.t();}
-
-  Matrix Mean;
-  Mean = mean(res);
-
-  for (int ctr = 1; ctr <= res.Nrows(); ctr++) {
-    for (int ctr2 =1; ctr2 <= res.Ncols(); ctr2++) {
-      res(ctr,ctr2)-=Mean(1,ctr2);
-    }
-  }
-  if (dim != 1) {res=res.t();}
+  Matrix res = mat;
+  remmean_econ(res,dim);
   res.Release();
   return res;
 }
 
-
-void remmean(const Matrix& mat, Matrix& demeanedmat, Matrix& Mean,  const int dim)
-{ 
-  if (dim == 1) {demeanedmat=mat;}
-  else {demeanedmat=mat.t();}
-
-  Mean = mean(demeanedmat);
-
-  for (int ctr = 1; ctr <= demeanedmat.Nrows(); ctr++) {
-    for (int ctr2 =1; ctr2 <= demeanedmat.Ncols(); ctr2++) {
-      demeanedmat(ctr,ctr2)-=Mean(1,ctr2);
-    }
-  }
-  if (dim != 1){demeanedmat = demeanedmat.t();Mean = Mean.t();}
-}
-
-ReturnMatrix cov(const Matrix& mat, const int norm)
+/* ReturnMatrix cov(const Matrix& mat, const int norm)
 { 
   SymmetricMatrix res;
   Matrix tmp;
@@ -1942,18 +2112,36 @@ ReturnMatrix cov(const Matrix& mat, const int norm)
   res.Release();
   return res; 
 }
+*/
+
+ReturnMatrix cov(const Matrix& mat, const int norm)
+{ 
+  SymmetricMatrix res;
+  res << ones(mat.Ncols(),mat.Ncols()); 
+
+  Matrix meanM;
+  int N;
+  meanM = mean(mat);
+  if (norm == 1) {N = mat.Nrows();}
+  else {N = mat.Nrows()-1;}  
+  for (int ctr=1; ctr <= mat.Nrows(); ctr++)
+  res << res + (mat.Row(ctr) - meanM ).t() * (mat.Row(ctr) - meanM);
+  res = res/N;
+
+  res.Release();
+  return res; 
+}
 
 ReturnMatrix corrcoef(const Matrix& mat, const int norm)
 { 
-  SymmetricMatrix res;
-  SymmetricMatrix C;
-  C = cov(mat,norm);
-  Matrix D;
-  D=diag(C);
-  D=pow(sqrt(D*D.t()),-1);
-  res << SP(C,D);
-  res.Release();
-  return res; 
+  	SymmetricMatrix res;
+	res = cov(mat,norm);
+	Matrix D;
+	D=diag(res);
+	D=sqrt(D*D.t());
+	res << SD(res,D);
+	res.Release();
+	return res;
 }
 
 ReturnMatrix flipud(const Matrix& mat)
@@ -2001,9 +2189,12 @@ void powerspectrum(const Matrix &Mat1, Matrix &Result, bool useLog)
       ColumnVector tmpPow;
       
       RealFFT(tmpCol,FtmpCol_real,FtmpCol_imag);
-      tmpPow = pow(FtmpCol_real,2)+pow(FtmpCol_imag,2);
+	  pow(FtmpCol_real,2);
+      pow(FtmpCol_imag,2);
+
+      tmpPow = FtmpCol_real+FtmpCol_imag;
       tmpPow = tmpPow.Rows(2,tmpPow.Nrows());
-      if(useLog){tmpPow = log(tmpPow);}
+      if(useLog){log(tmpPow);}
       if(res.Storage()==0){res= tmpPow;}else{res|=tmpPow;}
     }
     Result=res;
@@ -2195,9 +2386,16 @@ void ols(const Matrix& data,const Matrix& des,const Matrix& tc, Matrix& cope,Mat
 }
 
 float ols_dof(const Matrix& des){
+  if ( des.Nrows() > 4000 ) //Use the simple version as huge designs require too much RAM in the full calculation
+    return des.Nrows() - des.Ncols();
+  try {
   Matrix pdes = pinv(des);
   Matrix R=IdentityMatrix(des.Nrows())-des*pdes;
-  return R.Trace();
+  return R.Trace();}
+  catch (...) {
+    cerr << "ols_dof: Error in determining the trace, resorting to basic calculation" << endl;
+  }
+  return des.Nrows() - des.Ncols();
 }
 
 int conjgrad(ColumnVector& x, const Matrix& A, const ColumnVector& b, int maxit,
